﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace ConsoleApplication1
{
    class Program
    {
        static DateTime StartTime;

        static int N, M, P, C;
        static int[,] matrixGlob;

        static int[] dX = new int[] { 0, 0, -1, 1 };
        static int[] dY = new int[] { -1, 1, 0, 0 };

        static void shiftLeft(int[,] matrix, int r, int count)
        {
            int InZero = matrix[r, 0];
            int index = 0;

            for (int c = 0; c < M - 1; c++)
            {
                int nextIndex = (index + count) % M;
                matrix[r, index] = matrix[r, nextIndex];
                index = nextIndex;
            }
            matrix[r, index] = InZero;
        }

        static void shiftUp(int[,] matrix, int c, int count)
        {
            int InZero = matrix[0, c];
            int index = 0;

            for (int r = 0; r < N - 1; r++)
            {
                int nextIndex = (index + count) % N;
                matrix[index, c] = matrix[nextIndex, c];
                index = nextIndex;
            }
            matrix[index, c] = InZero;
        }

        static int Score(int[,] matrix)
        {
            int score = 0;

            int[,] mask = new int[N, M];
            for (int r = 0; r < N; r++)
            {
                for (int c = 0; c < M; c++)
                {
                    if (mask[r, c] == 0)
                    {
                        int currentCount = 0;

                        Queue<int> Q = new Queue<int>();

                        Q.Enqueue(r * M + c);
                        mask[r, c] = 1;

                        while (Q.Count > 0)
                        {
                            currentCount++;

                            int RC = Q.Dequeue();
                            int cY = RC / M;
                            int cX = RC % M;

                            for (int d = 0; d < 4; d++)
                            {
                                int y = cY + dY[d];
                                int x = cX + dX[d];

                                if (y >= 0 && y < N && x >= 0 && x < M)
                                {
                                    if (mask[y, x] == 0 && matrix[y, x] == matrix[cY, cX])
                                    {
                                        Q.Enqueue(y * M + x);
                                        mask[y, x] = 1;
                                    }
                                }
                            }
                        }

                        score += currentCount * currentCount;
                    }
                }
            }
            return score;
        }

        static Random rand = new Random(7 * 42 * 88 * 5);
        

        static List<string> findBest(int[,] matrix, int maximalP, out int ActualBestAsInt)
        {
            List<string> bestLog = new List<string>();
            List<int> bestLogScore = new List<int>();

            for (int operation = 0; operation < maximalP; operation++)
            {

                if (operation > 0 && (operation * N * M / 10000) % 10 == 0)
                {
                    if ((DateTime.Now - StartTime).TotalMilliseconds > 4500)
                    {
                        break;
                    }
                }

                int randomNumber = rand.Next();
                int way = randomNumber % 2;
                int xx = (randomNumber / 2) % M;
                int yy = (randomNumber / (2 * M)) % N;

                if (way == 1)
                {
                    if (xx == 0)
                        continue;
                    shiftLeft(matrix, yy, xx);
                    bestLog.Add("L " + (yy + 1) + " " + xx);
                }
                else
                {
                    if (yy == 0)
                        continue;
                    shiftUp(matrix, xx, yy);
                    bestLog.Add("U " + (xx + 1) + " " + yy);
                }

                bestLogScore.Add(Score(matrix));
            }

            int bestI = 0;
            for (int i = 1; i < bestLogScore.Count; i++)
            {
                if (bestLogScore[i] > bestLogScore[bestI])
                    bestI = i;
            }

            int bestAsInt = bestLogScore[bestI];

            List<string> ActualBestLog = new List<string>();
            for (int i = 0; i <= bestI; i++)
            {
                ActualBestLog.Add(bestLog[i]);
            }
            ActualBestAsInt = bestAsInt;


            //  return bestLog; // just for test again


            return ActualBestLog;

        }


        static void CheckOutput(int[,] matrix, List<string> output)
        {
            foreach (string s in output)
            {
                string[] command = s.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                if (command[0] == "U")
                {
                    shiftUp(matrix, int.Parse(command[1]) - 1, int.Parse(command[2]));
                }
                else if (command[0] == "L")
                {
                    shiftLeft(matrix, int.Parse(command[1]) - 1, int.Parse(command[2]));
                }
            }

            int score = Score(matrix);
            return;
        }

        static void Main(string[] args)
        {
            StartTime = DateTime.Now;

            using (StreamReader reader = new StreamReader("recycle.in"))
            {
                string NMPCstr = reader.ReadLine();
                string[] NMPC = NMPCstr.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                N = int.Parse(NMPC[0]);
                M = int.Parse(NMPC[1]);
                P = int.Parse(NMPC[2]);
                C = int.Parse(NMPC[3]);
                matrixGlob = new int[N, M];
                for (int r = 0; r < N; r++)
                {
                    string line = reader.ReadLine();
                    string[] numbers = line.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                    for (int c = 0; c < M; c++)
                    {
                        matrixGlob[r, c] = int.Parse(numbers[c]);
                    }
                }
            }

            //int[,] matrixOrg = new int[N, M];
            //for (int r = 0; r < N; r++)
            //{
            //    for (int c = 0; c < M; c++)
            //    {
            //        matrixOrg[r, c] = matrixGlob[r, c];
            //    }
            //}

            // int firstScore = Score(matrixGlob);

            int score;
            List<string> log = findBest(matrixGlob, P, out score);

            //List<string> fakeLog = new List<string>();
            //fakeLog.Add("U 2 3");
            //fakeLog.Add("L 1 1");
            //fakeLog.Add("L 4 1");

            //CheckOutput(matrixOrg, log);


            using (StreamWriter writer = new StreamWriter("recycle.out"))
            {
                writer.WriteLine(log.Count.ToString());
                for (int i = 0; i < log.Count; i++)
                {
                    writer.WriteLine(log[i]);
                }
            }

        }
    }
}
